#!/usr/bin/env python
"""Setup file for jellyfish."""

import os
import sys
from distutils.core import setup, Extension, Command
from distutils.command.build import build
from distutils import log

exts = [
    'jellyfishmodule.c', 'jaro.c', 'hamming.c', 'levenshtein.c',
    'damerau_levenshtein.c', 'mra.c', 'soundex.c', 'metaphone.c',
    'nysiis.c', 'porter.c']

class TestCommand(Command):
    """Command for running unittests without install."""

    user_options = [("args=", None, '''The command args string passed to
                                    unittest framework, such as 
                                     --args="-v -f"''')]

    def initialize_options(self):
        self.args = ''
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.run_command('build')
        bld = self.distribution.get_command_obj('build')
        #Add build_lib in to sys.path so that unittest can found DLLs and libs
        sys.path = [os.path.abspath(bld.build_lib)] + sys.path

        import shlex
        import unittest
        test_argv0 = [sys.argv[0] + ' test --args=']
        #For transfering args to unittest, we have to split args
        #by ourself, so that command like:
        #python setup.py test --args="-v -f"
        #can be executed, and the parameter '-v -f' can be
        #transfering to unittest properly.
        test_argv = test_argv0 + shlex.split(self.args)
        unittest.main(module=None, defaultTest='test.JellyfishTestCase', argv=test_argv)


cmdclass = {'test': TestCommand}

classifiers = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Text Processing :: Linguistic"],

with open('README.rst') as readme:
    long_description = readme.read()

setup(name='jellyfish',
      version='0.2.2',
      platforms=['any'],
      description=("a library for doing approximate and "
                   "phonetic matching of strings."),
      url='http://github.com/sunlightlabs/jellyfish',
      long_description=long_description,
      classifiers=classifiers,
      license='BSD',
      ext_modules=[Extension('jellyfish', exts)],
      cmdclass=cmdclass)



# -*- coding: utf-8 -*-
import csv
import unittest
import jellyfish


class JellyfishTestCase(unittest.TestCase):

    def test_jaro_winkler(self):
        cases = [("dixon", "dicksonx", 0.8133),
                 ("dixon", "dicksonx", 0.8133),
                 ("martha", "marhta", 0.9611),
                 ("dwayne", "duane", 0.84),
                 ]

        for (s1, s2, value) in cases:
            self.assertAlmostEqual(jellyfish.jaro_winkler(s1, s2), value,
                                   places=4)

    def test_jaro_distance(self):
        cases = [("dixon", "dicksonx", 0.767),
                 ("dixon", "dicksonx", 0.767),
                 ("martha", "marhta", 0.944),
                 ("dwayne", "duane", 0.822),
                 ]

        for (s1, s2, value) in cases:
            self.assertAlmostEqual(jellyfish.jaro_distance(s1, s2), value,
                                   places=3)

    def test_hamming_distance(self):
        cases = [("", "", 0),
                 ("", "abc", 3),
                 ("abc", "abc", 0),
                 ("acc", "abc", 1),
                 ("abcd", "abc", 1),
                 ("abc", "abcd", 1),
                 ("testing", "this is a test", 13),
                 ]

        for (s1, s2, value) in cases:
            self.assertEqual(jellyfish.hamming_distance(s1, s2), value)

    def test_levenshtein_distance(self):
        cases = [("", "", 0),
                 ("abc", "", 3),
                 ("bc", "abc", 1),
                 ("kitten", "sitting", 3),
                 ("Saturday", "Sunday", 3),
                 ]

        for (s1, s2, value) in cases:
            self.assertEqual(jellyfish.levenshtein_distance(s1, s2), value)

    def test_damerau_levenshtein_distance(self):
        cases = [("", "", 0),
                 ("abc", "", 3),
                 ("bc", "abc", 1),
                 ("abc", "acb", 1),
                 ]

        for (s1, s2, value) in cases:
            self.assertEqual(jellyfish.damerau_levenshtein_distance(s1, s2),
                             value)

    def test_soundex(self):
        cases = [("Washington", "W252"),
                 ("Lee", "L000"),
                 ("Gutierrez", "G362"),
                 ("Pfister", "P236"),
                 ("Jackson", "J250"),
                 ("Tymczak", "T522"),
                 ("", ""),
                 ("A", "A000"),
                 (u"Çáŕẗéř", "C636"),
                 ]

        for (s1, code) in cases:
            self.assertEqual(jellyfish.soundex(s1), code)

    def test_metaphone(self):
        cases = [("metaphone", 'MTFN'),
                 ("wHErE", "WR"),
                 ("shell", "XL"),
                 ("this is a difficult string", "0S IS A TFKLT STRNK"),
                 ("aeromancy", "ERMNS"),
                 ("Antidisestablishmentarianism", "ANTTSSTBLXMNTRNSM"),
                 ("sunlight labs", "SNLT LBS"),
                 ("sonlite laabz", "SNLT LBS"),
                 (u"Çáŕẗéř", "KRTR"),
                 ('kentucky', 'KNTK'),
                 ('KENTUCKY', 'KNTK'),
                 ]

        for (s1, code) in cases:
            self.assertEqual(jellyfish.metaphone(s1), code)

    def test_nysiis(self):
        cases = [("Worthy", "WARTY"),
                 ("Ogata", "OGAT"),
                 ("montgomery", "MANTGANARY"),
                 ("Costales", "CASTAL"),
                 ("Tu", "T"),
                 ("martincevic", "MARTANCAFAC"),
                 ("Catherine", "CATARAN"),
                 ("Katherine", "CATARAN"),
                 ("Katerina", "CATARAN"),
                 ("Johnathan", "JANATAN"),
                 ("Jonathan", "JANATAN"),
                 ("John", "JAN"),
                 ("Teresa", "TARAS"),
                 ("Theresa", "TARAS"),
                 ("Jessica", "JASAC"),
                 ("Joshua", "JAS"),
                 ("Bosch", "BAS"),
                 ("Lapher", "LAFAR"),
                 ]

        for (s1, s2) in cases:
            self.assertEqual(jellyfish.nysiis(s1), s2)

    def test_match_rating_codex(self):
        cases = [("Byrne", "BYRN"),
                 ("Boern", "BRN"),
                 ("Smith", "SMTH"),
                 ("Smyth", "SMYTH"),
                 ("Catherine", "CTHRN"),
                 ("Kathryn", "KTHRYN"),
                 ]

        for (s1, s2) in cases:
            self.assertEqual(jellyfish.match_rating_codex(s1), s2)

    def test_match_rating_comparison(self):
        cases = [("Bryne", "Boern", True),
                 ("Smith", "Smyth", True),
                 ("Catherine", "Kathryn", True),
                 ("Michael", "Mike", False),
                 ]

        for (s1, s2, value) in cases:
            self.assertEqual(jellyfish.match_rating_comparison(s1, s2), value)

    def test_match_rating_comparison_segfault(self):
        import hashlib
        sha1s = [hashlib.sha1(str(v).encode('ascii')).hexdigest() for v in range(100)]
        # this segfaulted on 0.1.2
        assert [[jellyfish.match_rating_comparison(h1, h2) for h1 in sha1s]
                for h2 in sha1s]

    def test_porter_stem(self):
        with open('porter-test.csv') as f:
            reader = csv.reader(f)
            for (a, b) in reader:
                self.assertEqual(jellyfish.porter_stem(a.lower()), b.lower())

if __name__ == '__main__':
    unittest.main()
